
#import vintools as v

def _create_gene_dict(genes, chrom, start, stop):

    """"""
    NotImplemented
    # GeneDict = v.ut.EmptyDict(genes)

    # for n, gene in enumerate(GeneDict.keys()):
    #     GeneDict[gene]["Chromosome"] = chrom[n]
    #     GeneDict[gene]["Start"] = start[n]
    #     GeneDict[gene]["End"] = stop[n]

    # return GeneDict