# _utilities __init__.py

__module_name__ = "__init__.py"
__author__ = ", ".join(["Michael E. Vinyard"])
__email__ = ", ".join(["vinyard@g.harvard.edu",])


#from ._funcs._get_gene_exons import _get_gene_df_from_gtf as get_gene
