class _GTF(object):
    
    def __init__(self, path):
        
        """"""
        
        self.file = open(path, "r")
        self.header={}
        
        
    def read(path):
        
        """"""
        
        ...